AC_DEFUN([LTMODEM_OS],[
	AC_MSG_CHECKING(OS)
	if test ! `uname -s` = "Linux"
	then
  		AC_MSG_RESULT("ERROR: This driver will only work on Linux")
  		exit 1
	fi
	tmp=`uname -s`	
	AC_MSG_RESULT($tmp)
])

AC_DEFUN([LTMODEM_PROZESSOR],[
	AC_MSG_CHECKING(machine type)
	if test -n `uname -m | grep 'i.86'` ; then
		tmp=`uname -m`	
  		AC_MSG_RESULT(Processor $tmp is supported.)
# elif test -n `uname -m | grep 'x86_64'` ; then
# echo "Processor `uname -m` is supported."
	else  
  		AC_MSG_RESULT(ERROR: This driver will only work on Intel iX86 and AMD processor PCs)
  		exit 1
	fi
])	

AC_DEFUN([LTMODEM_GCCVERSION],[
	GCC="yes"
	AC_PROG_CC
	AC_MSG_CHECKING(gcc major version)
	GCC_INPUT=`cat /proc/version | sed -e 's/)/ /g' | sed -e 's/(/ /g'`
# sed to remove  (   )
	GCC_PC=`for i in $GCC_INPUT ; do echo $i ; done | grep -A2 gcc | grep -A1 ersion | grep -v  ersion`
	GCC_TEST=`echo $GCC_PC | cut -d. -f1`
	echo $GCC_TEST > tmpm
	if grep "-" tmpm >/dev/null ; then
  		GCC_VERSION=`echo $GCC_TEST | cut -d- -f2`
	else
  		GCC_VERSION=$GCC_TEST
	fi
	AC_MSG_RESULT($GCC_VERSION)
	AC_SUBST(GCC_VERSION)
	rm tmpm
])

AC_DEFUN([LTMODEM_MODULENAMES],[
	AC_MSG_CHECKING(module names)
	KERNEL_MAJOR=`echo $KERNEL_VERSION | cut -d. -f1`
	KERNEL_MINOR=`echo $KERNEL_VERSION | cut -d. -f2`
	KERNEL_RELEASE=`echo $KERNEL_VERSION | cut -d. -f3-`
	case "$KERNEL_MAJOR.$KERNEL_MINOR" in
	2.2)
 		MODULEDIR="/lib/modules/$KERNEL_VERSION/misc"
		MODULESUFFIX=o
		DPKG=`ls -d /var/lib/dpkg 2>/dev/null`
 		;;
	2.[45])
 		MODULEDIR="/lib/modules/$KERNEL_VERSION/extra"
 		MODULESUFFIX=o
		DPKG=`ls -d /var/lib/dpkg 2>/dev/null`
 		;;
	2.[6])
 		MODULEDIR="/lib/modules/$KERNEL_VERSION/extra"
		MODULESUFFIX=ko
 		DPKG=`ls -d /var/lib/dpkg 2>/dev/null`
 		;;
	*)
 		AC_MSG_ERROR(configure support only for 2.2, 2.4, 2.5 and 2.6 kernels)
	esac
	AC_MSG_RESULT(done)
])

AC_DEFUN([LTMODEM_KERNELVERSION],[
	AC_MSG_CHECKING(for kernelversion in $1)
	KVERSION="conftest.kversion"
	CFLAGS="-I$1/include"
	AC_LINK_IFELSE(
#define __KERNEL__
#include "linux/autoconf.h"
#include "linux/config.h"
#include "linux/version.h"
#include "linux/utsrelease.h"
#undef __KERNEL__
#include <stdio.h>
#include <errno.h>
#include <string.h> 
				int main(int argc,char** argv){
  					FILE *of;
  					if ( (of=fopen("$KVERSION","w")) == NULL ){
						fprintf(stdout,"Error while writing kernel header version file: %s\n",strerror(errno));
    					return(1);
}
  					fprintf(of,"%s",UTS_RELEASE);
  					close(of);
  					return(0);
}
				, ./conftest && SOURCE_VERSION=`cat $KVERSION`
				, SOURCE_VERSION=" ")
	if test $SOURCE_VERSION
	then
		AC_MSG_RESULT($SOURCE_VERSION)
	else
		AC_MSG_RESULT(none)
	fi
])

AC_DEFUN([LTMODEM_KERNELDIR],[
	AC_ARG_WITH(kernel,
  				AS_HELP_STRING([--with-kernel=dir],[provide the directory with kernel headers]),
  					KERNEL_DIR="$withval", KERNEL_DIR="")
	if test ! -z $KERNEL_DIR
	then
		AC_MSG_CHECKING(for kernel headers)
  		if test ! -r $KERNEL_DIR/include/linux/version.h
  		then
    		AC_MSG_RESULT(ERROR: Kernel headers not found in $KERNEL_DIR or not readable)
    		exit 1
		else
			AC_MSG_RESULT($KERNEL_DIR)
			LTMODEM_KERNELVERSION($KERNEL_DIR)
  		fi	
	fi
])

AC_DEFUN([LTMODEM_KERNELEXTRAVERSION],[
	AC_MSG_CHECKING(for kernel extra version)
	if test -f $KERNEL_DIR/.extraversion
# needed for proper installation
	then
    	EXTRAVERSION=`cat $KERNEL_DIR/.extraversion`
	else
   		Ktest1=`echo $KERNEL_VERSION | cut -d- -f1`
   		Ktest2=`echo $Ktest1 | cut -d. -f4`
   		if test -n "$Ktest2" 
   		then
      		EXTRAVERSION=.$Ktest2
  		else
     		EXTRAVERSION=-`echo  $KERNEL_VERSION | cut -d- -f2-`
  		fi
	fi
	AC_MSG_RESULT($EXTRAVERSION)	
])

AC_DEFUN([LTMODEM_MODPOSTSYM],[
	AC_MSG_CHECKING(for modpost configuration)
# modpost's symverfile configuration
	SYMVERFILE=
	KERNEL_RELEASE_NUM=`echo $KERNEL_RELEASE | sed 's/^\([[0-9]][[0-9]]*\).*/\1/'`
	if test "$KERNEL_MAJOR.$KERNEL_MINOR" = "2.6" -a $KERNEL_RELEASE_NUM -ge 5 
	then
    	if test ! -f $KERNEL_DIR/Module.symvers 
		then
        	AC_MSG_WARN(No $KERNEL_DIR/Module.symvers file) 
    	fi
    	SYMVERFILE=$KERNEL_DIR/Module.symvers 
	fi
	EXTMOD_SWITCH=SUBDIRS
	if test "$KERNEL_MAJOR.$KERNEL_MINOR" == "2.6" -a $KERNEL_RELEASE_NUM -ge 6 
	then
    	EXTMOD_SWITCH=M
	fi
	AC_SUBST(EXTMOD_SWITCH)
	AC_MSG_RESULT(done)
])	


AC_DEFUN([LTMODEM_KERNELPACKAGE],[
	AC_MSG_CHECKING(for standard installed kernel packages)
	if ! rpm -q kernel >/dev/null ; then
		AC_MSG_RESULT([No kernel package])
		AC_MSG_CHECKING([test for package that provides the kernel (SuSE)])
		K2=`rpm -q --whatprovides "--queryformat=%{name}" $KPKG`
		test -n "$K2" && REQUIRES_KERNEL="$K2 = $KERNEL_VERSION"
		AC_MSG_RESULT([$K2])
	else
		REQUIRES_KERNEL="kernel = $KERNEL_VERSION"
		AC_MSG_RESULT(found)
	fi
])

AC_DEFUN([LTMODEM_IRQSPINLOCK],[
	AC_MSG_CHECKING(for irq_spinlock in struct async_struct)
	CFLAGS="-I$KERNEL_DIR/include -D__KERNEL__ -DMODULE -DLINUX\
	-I$KERNEL_DIR/include/asm/mach-default"
	AC_LINK_IFELSE(
#include "$KERNEL_DIR/include/linux/autoconf.h"
#include "$KERNEL_DIR/include/linux/config.h"
#include "$KERNEL_DIR/include/linux/module.h"
#include "$KERNEL_DIR/include/linux/types.h"
#include "$KERNEL_DIR/include/linux/spinlock.h"
#include "$KERNEL_DIR/include/linux/serial.h"
#include "$KERNEL_DIR/include/linux/serialP.h"
					int main(int argc,char** argv){
						struct async_struct info;
						info.irq_spinlock;
						return(0);
}
					, HAS_IRQSPINLOCK=1
					, HAS_IRQSPINLOCK=0)
	AC_MSG_RESULT(yes)
	AC_SUBST(HAS_IRQSPINLOCK)
])

AC_DEFUN([LTMODEM_PCMCIA],[
	AC_ARG_ENABLE(pcmcia,
				AS_HELP_STRING([--enable-pcmcia],[compile PCMCIA version of driver]),
				PCMCIA=1, PCMCIA=0)
	AC_ARG_WITH(pcmciasrc,
				AS_HELP_STRING([--with-pcmciasrc=dir],[location of pcmcia-cs source]),
				PCMCIASRC=$withval,PCMCIASRC='')
	if(test "$PCMCIA" = "1" -o "$FOLDER" = "pcmcia")
	then	
		AC_MSG_CHECKING(for kernel PCMCIA)
		if(test "$CONFIG_PCMCIA"="y" || test "$CONFIG_PCMCIA"="m")
		then
			KERNEL_PCMCIA=1
			AC_MSG_RESULT(found)
		else
			KERNEL_PCMCIA=0
			AC_MSG_RESULT(not found)
		fi
		if(test "$KERNEL_PCMCIA" = "0")
		then
			AC_MSG_CHECKING(for pcmcia-cs source)
            PCMCIASRC=$PCMCIA_SRC
			if( test -f $PCMCIASRC/include/pcmcia/ds.h)
			then
				CS_PCMCIA=1
				AC_MSG_RESULT(found)
			else
				CS_PCMCIA=0
				AC_MSG_RESULT(none)
			fi
        else
          CS_PCMCIA=0  
		fi
		if (test "$KERNEL_PCMCI" -o "$CS_PCMCIA")
        then
          PCMCIA=1
        fi
		if(test "$PCMCIA" = "1")
		then
			LT_MODULE_OBJ="martian_drv_cs.o"
            LT_MODULE_KOBJ="martian_drv_cs.$MODULESUFFIX pcmcia" 
			LT_MODEM_OBJ="martian_cs.o martian.o marsio.o mfifo.o"
			LT_OBJ_NAME="martian_drv_cs-objs = "
			PCMCIA_SRC="-DPCMCIA"	
            PCMCIADIR_SPEC="%define pcmciadir /etc/pcmcia"
			INSTALL_PCMCIADIR="mkdir -p \$RPM_BUILD_ROOT/etc/pcmcia"
			RPM_TARGET="pcmcia"
		fi
	else
		LT_MODULE_OBJ="martian_drv.o"
        LT_MODULE_KOBJ="martian_drv.$MODULESUFFIX" 
		LT_OBJ_NAME="martian_drv-objs = "
		LT_MODEM_OBJ="martian.o marsio.o mfifo.o"
		PCMCIA_SRC="-UPCMCIA"
        PCMCIADIR_SPEC=""
		INSTALL_PCMCIADIR=""
		RPM_TARGET="pci"
	fi
	AC_SUBST(PCMCIA_SRC)
	AC_SUBST(LT_MODULE_OBJ)
    AC_SUBST(LT_MODULE_KOBJ) 
	AC_SUBST(LT_OBJ_NAME)
	AC_SUBST(LT_MODEM_OBJ)
    AC_SUBST(PCMCIADIR_SPEC) 
	AC_SUBST(INSTALL_PCMCIADIR)
	AC_SUBST(PCMCIA)
	AC_SUBST(RPM_TARGET)
])

AC_DEFUN([LTMODEM_PCMCIADEBUG],[
	AC_ARG_ENABLE(pcmcia-debug,
			      AS_HELP_STRING([--enable-pcmcia-debug],[enable debug code in driver]),
				 PCMCIA_DEBUG="-DPCMCIA_DEBUG", PCMCIA_DEBUG="-UPCMCIA_DEBUG")
	AC_SUBST(PCMCIA_DEBUG)
])

AC_DEFUN([LTMODEM_PCMCIAINFO],[
	if( test $KERNEL_MAJOR -eq 2 && test $KERNEL_MINOR -eq 6 && test $KERNEL_RELEASE_NUM -gt 12)
	then
		AC_PATH_PROG([PCCARDCTL],[pccardctl],[/bin/false],[/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin])
	else
		AC_PATH_PROG([PCCARDCTL],[cardctl],[/bin/false],[/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin])
	fi
	AC_MSG_NOTICE([PCMCIA utilyties found])
	K2=`$PCCARDCTL ident`
	if ( ! test -z "$K2" )
	then
		if `$PCCARDCTL ident | grep 'product info:' >/dev/null`
		then
			PROD=`$PCCARDCTL ident | grep 'product info:'`
			if( ! test -z "$PROD")
			then
				PROD1=`echo $PROD | cut -f3 -d" " | sed -e "s/\"//g" | sed -e "s/,//g"`
				PROD2=`echo $PROD | cut -f2 -d"," | sed -e "s/\"//g" | sed -e "s/,//g"`
				PROD3=`echo $PROD | cut -f3 -d"," | sed -e "s/\"//g" | sed -e "s/,//g"`
				PROD4=`echo $PROD | cut -f4 -d"," | sed -e "s/\"//g" | sed -e "s/,//g"`
			fi
			MANF=`$PCCARDCTL ident | grep 'manfid:'`
			if( ! test -z "$MANF")
			then
				MANFID=`echo $MANF | cut -f2 -d" "| sed -e "s/,//g"`
				PRODID=`echo $MANF | cut -f3 -d" "| sed -e "s/,//g"`
			fi
		fi
	else
		AC_MSG_NOTICE([pcmcia-utils package not installed! Install it and rerun configure.])
		AC_MSG_ERROR([Exiting now!])
	fi
])