
#ifndef __KMARTIAN_H
#define __KMARTIAN_H

#include <linux/pci.h>		// pci_driver
#include <linux/proc_fs.h>	/*** proc entry */
#include <linux/wait.h>		/*** wait_queue */
#include <asm/atomic.h>

#include "martian.h"
#include "fifo.h"
#include "mfifo.h"

#define MPREFIX			__stringify (KBUILD_MODNAME)

#define MINFO(fmt...)		printk (KERN_INFO MPREFIX ": " fmt)
#define MERROR(fmt...)		printk (KERN_ERR MPREFIX ": " fmt)

#define MARTIAN_DEBUG

#ifdef MARTIAN_DEBUG
#  define MDEBUG(fmt...) 	printk (KERN_DEBUG MPREFIX ": " fmt)
#  define ASSERT(cond, action)	do {if (! (cond)) { action }} while (0)
#else
#  define MDEBUG(arg...)	do {} while (0)	
#  define ASSERT(cond, action) 	do {} while (0)
#endif 	

struct martian_metrics {
	unsigned BaseAddress;
	unsigned CommAddress;

	unsigned int irq;

	unsigned short vendor, device;
	unsigned short subsystem_vendor;
	unsigned short subsystem_device;
	unsigned flags;
	// void *

};

struct martian {
	struct pci_dev *dev;
	struct proc_dir_entry *entry;
	unsigned long state;
	
	wait_queue_head_t wq;	/* to wait in read/poll */

	/* isr data */
	mfifo_t dce_tx;
	mfifo_t dce_rx;
	mfifo_t pdm_rx;

	atomic_t count; 	/* interrupt counter */

	/* stats */
	struct irqcount icounts;

	/* resources */

	/* common data pointer */
	struct martian_common *common;
	int index;

	// configuration info
	struct martian_metrics metrics;
};

#define MARTIAN_STATE_OPEN	0
#define KMSTATE_UREADY		1 
#define KMSTATE_FIFO_FAULT	2 

#define KMSETTING_SERVE_IRQS	16 
#define KMSETTING_TRUESMP	17 
#define KMSETTING_DEBUG		18

#endif	/* __KMARTIAN_H */
