/* martian.h: 
 *	modem program/module common data structure
 *	shared bamil_d7, BaseAddressIndex, BaseAddress

 *	program to module connection ioctl
 */

#ifndef __MARTIAN_DRIVERHELPER_COMMON
#define __MARTIAN_DRIVERHELPER_COMMON

#include "mixspinlock.h"
#ifndef __KERNEL__
#define LOCK	"lock; "
typedef struct { volatile int counter; } atomic_t;
#else
#include <asm/atomic.h>
#endif

/* those types were originally in */
/*
typedef unsigned char byte;	
typedef unsigned short word;
typedef unsigned dword;
*/

/* martian program/module shared structure, 
 * mapped into user space via mmap call on device file
 */
#define MARTIAN_SIGNATURE	"Martian" 	// signature at the page start
#define MARTIAN_TSIGNATURE	"martiaN" 	/* terminating signature */
#define DECLARE_FIFO(type, fifo, size, bname)	\
	type fifo##_##bname[size];		\
	__u32 fifo##_wptr, fifo##_rptr;		\
	__u32 fifo##_buff_um;

struct martian_common {
	char signature[sizeof MARTIAN_SIGNATURE];

	/*** core relocated symbols ***/
	__u16	BaseAddressIndex;
	__u16	BaseAddressData;
	__u8 	BaseValue;
	__u8	dp_bamil_rd7;
	__u8	x_dsp_mars;

	__u16	BaseAddress;
	__u16	BaseAddress2;

	DECLARE_FIFO (__u16, io_dce_tx, 0x400, buff);
	DECLARE_FIFO (__u16, io_dce_rx, 0x800, buff);
	DECLARE_FIFO (__u8,  io_pdm_rx, 0x4b0, buff);

	__u16 V34Mode;
	__u8 S[0x12c];
	__u8 dp_dsp_data_in_progress;
	__u16 init_RBS_tick_flag;
	__u8 RBS_tick_rx;

	/*** non-core common ***/
	mspinlock_t lock __attribute__ ((aligned(4)));
	
	__u8 	rd7_isr;

	__u32	OverlappedIrqs; 	/* those came in the middle previous 
					 * were served in UM 
					 */
	__u32	RepeatedIrqs; 	        /* those came before previous 
					 * were served in UM 
					 */
	atomic_t UnservedIrqs;
	atomic_t UnpassedIrqs;

	int rx_overrun, pdm_rx_overrun;
	int rx_incorrespondence, tx_incorrespondence;
	int pdm_rx_incorrespondence;

	int rx, tx, pdm_rx;
	int dual_port_rx, dual_port_tx, io_dual_port_rx;

	int dirty, pdm_dirty;
	__u32 latest_rptr, latest_wptr;

	char kstamp[9];
	char tsignature[sizeof MARTIAN_TSIGNATURE];
};

#define MCB_PAGES	((PAGE_SIZE - 1 + sizeof (struct martian_common)) >> PAGE_SHIFT)

struct martian_conf {
	__u16 BaseAddress;
	__u16 CommAddress;

	__u8 irq;

	unsigned short vendor, device;
	unsigned subsys_vendor;
	unsigned subsys_device;
	unsigned flags;
	// void *

	int index; 	// device index; to find slot in ; can be derived
};


#define IR			0xd7
#define IR_SIGNIFICANT		0x0e

#define MARTIAN_DEV_TOSHIBA	0x00000001

struct irqcount {
	int b, f, n, i, o;
};

struct martian_irq_state {
	char string[11];
	struct irqcount counts;
	int count;
};


#define MARTIAN_CONF_GET 	_IOR ('S', 0, struct martian_conf)
#define MARTIAN_IRQ_STATE 	_IOR ('S', 1, struct martian_irq_state)
#define MARTIAN_DUMP_VARS 	_IO  ('S', 2)
#define MARTIAN_SERVE_IRQS 	_IO  ('S', 3)
#define MARTIAN_MODEM_READY 	_IO  ('S', 4)
#define MARTIAN_CHECK_SMP 	_IOW ('S', 5, int)
#define MARTIAN_TRUE_SMP 	_IO  ('S', 6)
#define MARTIAN_SCREWUP_DCE_RX 	_IO  ('S', 7)
#define MARTIAN_FIFO_FAULT 	_IO  ('S', 8)
#define MARTIAN_KDEBUG	 	_IO  ('S', 9)

#undef DECLARE_FIFO	/* static */

#endif /* __MARTIAN_DRIVERHELPER_COMMON */

