#include <asm/io.h>

#include "mixspinlock.h"
#include "kmartian.h"
#include "martian.h"

u8 mars_read_register (struct martian_common *common, u8 reg)
{
	outb (  reg,

		(reg < 0x80) || ((reg >= 0xa0) && (reg < 0xd0)) ? 
		common->BaseAddress : 
		common->BaseAddress2
	);

	return inb (common->BaseAddressData);
}

void mars_write_register (struct martian_common *common, u8 reg, u8 val) 
{
	outb (reg, common->BaseAddress);
	return outb (val, common->BaseAddressData);
}

void mars_write_word (struct martian_common *common, u16 addr, u16 val)
{
	addr = common->BaseAddress + addr;
	outw (val, addr);
}

#define CHECK_FIFO_CORRESPOND_HEAD(fifo) \
({	\
	int incor = 0;							\
	int size = sizeof mcb->io_##fifo##_buff 			\
		/ sizeof mcb->io_##fifo##_buff[0];			\
	int uhead = (mcb->io_##fifo##_wptr - mcb->io_##fifo##_buff_um)	\
		/ sizeof mcb->io_##fifo##_buff[0];			\
	int utail = (mcb->io_##fifo##_rptr - mcb->io_##fifo##_buff_um)	\
		/ sizeof mcb->io_##fifo##_buff[0];			\
	if (uhead != mdev->fifo.head) { 				\
		mdev->fifo.head = uhead;				\
		incor = 1;						\
	}								\
	(!incor) && (utail >= 0) && (utail < size);	\
})

#define KFIFO_UPDATE_TAIL(fifo)						\
mdev->fifo.tail = (mcb->io_##fifo##_rptr - mcb->io_##fifo##_buff_um) 	\
			/ sizeof mcb->io_##fifo##_buff[0];

#define UFIFO_UPDATE_HEAD(fifo)						\
mcb->io_##fifo##_wptr = mcb->io_##fifo##_buff_um + 			\
	(sizeof mcb->io_##fifo##_buff[0]) * mdev->fifo.head;

static inline unsigned char mdual_port_rx (struct martian *mdev) {
	struct martian_common *mcb = mdev->common;

	register __u16 *startptr;
	__u8 reg, rb0;
	int num;

	mcb->dual_port_rx++;

	rb0 = mars_read_register (mcb, 0xb0);
	num = rb0 & 0x1f;
	if (num == 0)
		return mars_read_register (mcb, 0xb0);

	ASSERT (CHECK_FIFO_CORRESPOND_HEAD (dce_rx), 
			mcb->rx_incorrespondence++;);

	KFIFO_UPDATE_TAIL (dce_rx);

	mcb->rx += num;
	startptr = (u16 *) mfifo_head (&mdev->dce_rx); 
	reg = 0x90; 
	
	while (--num != -1) {
		u16 val = mars_read_register (mcb, reg++);
		/* on BE should be transformed to LE */
		val = cpu_to_le16 (val);
		mfifo_put (&mdev->dce_rx, &val);

		if (mfifo_empty (&mdev->dce_rx))
			mcb->rx_overrun++;
	}

	/* update modem's */
	UFIFO_UPDATE_HEAD (dce_rx);

	
	if (rb0 & 0x20)
		*startptr |= cpu_to_le16 (0x4000);

	if ((rb0 & 0xc0) && ! mfifo_empty (&mdev->dce_rx)) {

		u16 *lastptr = (u16 *) mfifo_lastwr (&mdev->dce_rx);

		if (rb0 & 0x80) 
			*lastptr |= cpu_to_le16 (0x8000);

		if (rb0 & 0x40) 
			*lastptr |= cpu_to_le16 (0xa800);
	}

	// ASSERT (mfifo_ok (&mdev->dce_rx), mcb->kfifo_rx_bad++;);

	return mars_read_register (mcb, 0xb0);
}

static __u8 Last_rbs_tick; 
static inline int mio_dual_port_rx (struct martian *mdev)
{
	struct martian_common *mcb = mdev->common;
	u8 rb0 = mars_read_register (mcb, 0xb0);
	u8 reg = 0x90; 
	u8 num = 16;

	u32 io_dce_rx_rptr = mcb->io_dce_rx_rptr;
	u32 io_dce_rx_wptr = mcb->io_dce_rx_wptr;

	if (mcb->init_RBS_tick_flag) {
		rb0 &= 0x7;
		mcb->RBS_tick_rx = rb0;
		mcb->init_RBS_tick_flag = 0;
	}
	else if ((Last_rbs_tick + 4) % 6 != rb0)
		return mars_read_register (mcb, 0xb0);

	Last_rbs_tick = rb0;

	ASSERT (CHECK_FIFO_CORRESPOND_HEAD (pdm_rx), 
			mcb->pdm_rx_incorrespondence++;);

	KFIFO_UPDATE_TAIL (pdm_rx);

	do {
		u8 val = mars_read_register (mcb, reg++);
		mfifo_put (&mdev->pdm_rx, &val);

		/* if (mfifo_full (&mcb->pdm_rx)) */
		if (mfifo_empty (&mdev->pdm_rx))
			mcb->pdm_rx_overrun++;
	} while (--num);

	UFIFO_UPDATE_HEAD (pdm_rx);

	if (test_bit (KMSETTING_DEBUG, &mdev->state)) {
		if (io_dce_rx_rptr != mcb->io_dce_rx_rptr) {
			mcb->pdm_dirty++;
			mcb->latest_rptr = mcb->io_dce_rx_rptr;
		}

		if (io_dce_rx_wptr != mcb->io_dce_rx_wptr) {
			mcb->pdm_dirty += 0x1000;
			mcb->latest_wptr = mcb->io_dce_rx_wptr;
		}

		if ((io_dce_rx_wptr != mcb->io_dce_rx_wptr) || (io_dce_rx_wptr != mcb->io_dce_rx_wptr)) 
			set_bit (KMSTATE_FIFO_FAULT, &mdev->state);
	}

	mcb->io_dual_port_rx++;
	mcb->pdm_rx += (16 - num);

	return mars_read_register (mcb, 0xb0);
}

#define CRL_BYTE(w)	((w) >> 8)
#define CHECK_FIFO_CORRESPOND_TAIL(fifo) \
({	\
	int incor = 0;							\
	int size = sizeof mcb->io_##fifo##_buff				\
		/ sizeof mcb->io_##fifo##_buff[0];			\
	int uhead = (mcb->io_##fifo##_wptr - mcb->io_##fifo##_buff_um)	\
		/ sizeof mcb->io_##fifo##_buff[0];			\
	int utail = (mcb->io_##fifo##_rptr - mcb->io_##fifo##_buff_um)	\
		/ sizeof mcb->io_##fifo##_buff[0];			\
	if (utail != mdev->fifo.tail) { 				\
		mdev->fifo.tail = utail;				\
		incor = 1;						\
	}								\
	(!incor) && (uhead >= 0) && (uhead < size);	\
})

#define KFIFO_UPDATE_HEAD(fifo)						\
mdev->fifo.head = (mcb->io_##fifo##_wptr - mcb->io_##fifo##_buff_um) 	\
			/ sizeof mcb->io_##fifo##_buff[0];

#define UFIFO_UPDATE_TAIL(fifo)						\
mcb->io_##fifo##_rptr = mcb->io_##fifo##_buff_um + 			\
	(sizeof mcb->io_##fifo##_buff[0]) * mdev->fifo.tail;

#define	CF_CRL_BYTE(w)	(le16_to_cpu (w) >> 8)
#define	CF_DATA_BYTE(w)	(le16_to_cpu (w) & 0xff)

static inline void mdual_port_tx (struct martian *mdev)
{
	struct martian_common *mcb = mdev->common;

	unsigned char reg;
	unsigned char crl, num;

	mcb->dual_port_tx++;

	mars_read_register (mcb, 0x32);

	reg = 0x80;
	
	crl = CF_CRL_BYTE (*(u16 *) mfifo_tail (&mdev->dce_tx));
	crl &= (crl & 0x20) ? 0x20 : 0x40;


	ASSERT (CHECK_FIFO_CORRESPOND_TAIL (dce_tx), 
			mcb->tx_incorrespondence++;);

	KFIFO_UPDATE_HEAD (dce_tx);

	while (reg <= 0x85 && ! mfifo_empty (&mdev->dce_tx)) {
		u16 val;
		mfifo_get (&mdev->dce_tx, &val);
		mars_write_register (mcb, reg++, CF_DATA_BYTE(val));

		if (CF_CRL_BYTE (val) & 0x80) {
			crl |= 0x80;
			break;
		}

		if (0x40 & CF_CRL_BYTE (* (u16 *) mfifo_tail (&mdev->dce_tx))) 
			break;
	}

	UFIFO_UPDATE_TAIL (dce_tx);

	num = reg - 0x80; 
	if (num != 0) {
		crl |= num;
		mcb->tx += num;

		mcb->dp_dsp_data_in_progress = 1;
		mars_write_register (mcb, 0xb0, crl);
	}
	else {
		mcb->dp_dsp_data_in_progress = 0;
		mars_write_register (mcb, 0xb7, 1);
	}

	// ASSERT (mfifo_ok(&mdev->dce_tx), mcb->kfifo_tx_bad++;);
}

int process_stream (struct martian *mdev) {
	struct martian_common *mcb = mdev->common;
	u32 io_dce_rx_rptr = mcb->io_dce_rx_rptr;
	u32 io_dce_rx_wptr = mcb->io_dce_rx_wptr;

	if (mars_read_register (mcb, 0xb6) & 0x20) 
		return 0;

	if (mars_read_register (mcb, 0xb6) & 0x10) {
		if (mcb->S[0x6b] == 1 && ! mcb->V34Mode && mcb->S[0x96] == 0)
			/* return 0; */
			mio_dual_port_rx (mdev);
		else {
			mdual_port_rx (mdev);
			io_dce_rx_wptr = mcb->io_dce_rx_wptr;
		}
	}

	if (mars_read_register (mcb, 0xb6) & 0x1) 
		mdual_port_tx (mdev);

	if (test_bit (KMSETTING_DEBUG, &mdev->state)) {
		if (io_dce_rx_rptr != mcb->io_dce_rx_rptr) {
			mcb->dirty++;
			mcb->latest_rptr = mcb->io_dce_rx_rptr;
		}

		if (io_dce_rx_wptr != mcb->io_dce_rx_wptr) {
			mcb->dirty += 0x1000;
			mcb->latest_wptr = mcb->io_dce_rx_wptr;
		}

		if ((io_dce_rx_wptr != mcb->io_dce_rx_wptr) || (io_dce_rx_wptr != mcb->io_dce_rx_wptr)) 
			set_bit (KMSTATE_FIFO_FAULT, &mdev->state);
	}

	return 1;
	/*
	unsigned char reg = 0xb3; 
	do {
		DCPBufferStart[DCPwIndex++] = dp_int_regread (reg);

		if (DCPwIndex > sizeof DCPBuffer - 1)
			DCPwIndex = 0;

	} while (--reg != 0xb0);
	*/
}
