/*   core_if.c:
 *	linux user mode interface for mars core
 *
 */

#include <stdio.h>
#include <sys/io.h>

#include "sysdep.h"
#include <asm/types.h>

/* Mcb */
#include "mixspinlock.h"
#include "common.h"
#include "martian.h"


/* gives errors on suse 10.1 */
/* #include <asm/system.h> */ 

#ifndef local_irq_disable
#  define local_irq_disable() asm volatile ("cli")
#endif
#ifndef local_irq_enable
#  define local_irq_enable() asm volatile ("sti")
#endif

#include <sys/pci.h>
#include <sys/time.h>

#include "log.h"

// stubs for pci routines
int lt_pci_present() {
	return 1;
}

#define EMIT_PCI_FUNC_COUPLE(dim, type) \
	int lt_pcibios_read_config_##dim (__u8 bus, __u8 dev_fn, __u8 where, type *val) \
	{ return 0; } \
	int lt_pcibios_write_config_##dim (__u8 bus, __u8 dev_fn, __u8 where, type *val) \
	{ return 0; } 

EMIT_PCI_FUNC_COUPLE (byte, __u8);
EMIT_PCI_FUNC_COUPLE (word, __u16);
EMIT_PCI_FUNC_COUPLE (dword, __u32);

void lt_init_timer (void ) {
	// initialize timer
	LOGDEBUG(2,"init vi timer\n");
}

extern mtimer_t vitimer;
void lt_add_timer(void (*timerfunction)(unsigned long)) {
	const struct timespec delay = {
		.tv_sec = 0,
		.tv_nsec = 10 * 1000000
	};
	const struct itimerspec vi_delay = {
		.it_value = delay,
		.it_interval = {0, 0}
	};

	int res = vitimer.settime (vitimer.id, 0, &vi_delay, NULL);
	if (res < 0) {
		LOGERR ("start timer error %d\n", res);
	}
}

void lt_add_timer_old(void (*timerfunction)(unsigned long)) {
	struct itimerval next_pulse;
	/*extern int pulse_core;
	pulse_core = 0;*/

	/* printf ("a"); fflush (stdout); */

	next_pulse.it_value.tv_sec = 0;
	next_pulse.it_value.tv_usec = 10000;

	next_pulse.it_interval.tv_sec = 0;
	next_pulse.it_interval.tv_usec = 0;

	setitimer (ITIMER_REAL, &next_pulse, NULL);
}


unsigned VMODEM_Get_System_Time(void)
{
        struct timeval time;

        gettimeofday (&time, NULL);
        return time.tv_usec / 1000 + time.tv_sec * 1000;
}

#ifdef SMP_SUPPORT
void timertick (void *coretimertick) {
	queue_request();
}
#endif


// memory

/***** IO functions *****/
/* those are not stubs
 * those are principle
 */
extern __u8 dp_regread_nonint (__u8 reg);
extern void dp_regwrite_nonint (__u8 reg, __u8 value); 

__u8 dp_regread (__u8 reg) {
	// cli & (for smp) spinlock 
	local_irq_disable();
	if (config.true_smp)
		mspin_lock (&Mcb->lock);
	
	int ret = dp_regread_nonint (reg);

	// sti & unlock
	if (config.true_smp)
		mspin_unlock (&Mcb->lock);
	local_irq_enable();
	return ret;
}

void dp_regwrite (__u8 reg, __u8 value) {
	// cli & (for smp) spinlock 
	local_irq_disable();
	if (config.true_smp)
		mspin_lock (&Mcb->lock);

	dp_regwrite_nonint (reg, value);

	// sti & unlock
	if (config.true_smp)
		mspin_unlock (&Mcb->lock);
	local_irq_enable();
}


// low level io
__u8 inp (__u16 addr) {
	return inb (addr);
}

__u16 inpw (__u16 addr) {
	return inw (addr);
}

__u32 inpd (__u16 addr) {
	return inl (addr);
}


void outp (__u16 addr, __u8 value) {
	outb (value, addr);
}

void outpw (__u16 addr, __u16 value) {
	outw (value, addr);
}

void outpd (__u16 addr, __u32 value) {
	return outl (value, addr);
}

/**************************/



/* */
unsigned char Get_PCI_INTERRUPT_LINE(void) {
	return PCI_INTERRUPT_LINE;
}

unsigned char Get_PCI_BASE_ADDRESS_1 (void) {
	return PCI_BASE_ADDRESS_1;
}

unsigned char Get_PCI_BASE_ADDRESS_2 (void) {
	return PCI_BASE_ADDRESS_2;
}

unsigned Get_PCI_BASE_ADDRESS_IO_MASK (void) {
	return PCI_BASE_ADDRESS_IO_MASK;
}

unsigned char Get_PCI_BASE_ADDRESS_SPACE_IO (void) {
	return PCI_BASE_ADDRESS_SPACE_IO;
}

///
int function_1(int (*fn)(void *), char *a) {return 0;}
void lin_interruptible_sleep_on(void) {}
int lin_signal_pending(void) {return 0;}
void lin_kill(void) {}
void lin_wake_up(void) {}
int function_2(char *p) {return 0;}


struct lt_pci_dev_info;
int lt_pci_find_device(struct lt_pci_dev_info *lt_dev, unsigned int id, unsigned int num)
{
	LOGERR ("Unexpected call lt_pci_find_device\n");
	return 0;
}

