#include <asm/types.h>
#include "mixspinlock.h"
#include "martian.h"
#include "common.h"
#include "log.h"

#ifndef local_irq_disable
#  define local_irq_disable() asm volatile ("cli")
#endif
#ifndef local_irq_enable
#  define local_irq_enable() asm volatile ("sti")
#endif

static __inline__ void atomic_dec(atomic_t *v)
{
	__asm__ __volatile__(
		LOCK "decl %0"
		:"=m" (v->counter)
		:"m" (v->counter));
}

extern struct _stats {
	int cring, cstream, cbit3;
	int dual_port_tx, dual_port_rx, io_dual_port_rx;
	int tx, rx, pdm_rx;
	int dcp;
} stats;

struct core_stats core;

unsigned char *rd7_isr;

extern __u8 dp_int_regread (__u8 reg);
extern void dp_int_regwrite (__u8 reg, __u8 value); 

extern __u8 dp_bamil_rd7;
	
#define DECLARE_FIFO(type, fifo) 	\
extern type *fifo##_rptr, *fifo##_wptr;	\
extern type *fifo##_sptr, *fifo##_eptr

#define DECLARE_FIFO_W(type, fifo) 	\
extern type *fifo##_wptr;		\
extern type *fifo##_sptr, *fifo##_eptr

#define FIFO_NEXT(fifo)					\
		if (++fifo##_rptr >= fifo##_eptr) 	\
			fifo##_rptr = fifo##_sptr

#define FIFO_WRITE(fifo, val)				\
		*fifo##_wptr++ = (val); 		\
		if (fifo##_wptr >= fifo##_eptr) 	\
			fifo##_wptr = fifo##_sptr

#define FIFO_FULL(fifo) 	(fifo##_wptr == fifo##_rptr)
#define FIFO_CUR(fifo)		*fifo##_rptr
#define FIFO_DATA(fifo)		* (unsigned char *) fifo##_rptr
#define FIFO_CRL(fifo)		* (1 + (unsigned char *) fifo##_rptr)
	
static __u8 Last_rbs_tick; //bss[4]
static inline int mio_dual_port_rx (void )
{
	extern __u16 init_RBS_tick_flag;
	extern __u8 RBS_tick_rx;
	DECLARE_FIFO_W (__u8, io_pdm_rx);
	__u8 rb0 = dp_int_regread (0xb0); 
	__u8 reg = 0x90; 
	__u8 num = 16;

	stats.io_dual_port_rx++;

	if (init_RBS_tick_flag == 0x0) {
		if ((Last_rbs_tick + 4) % 6 != rb0) 
			return dp_int_regread (0xb0);
	}
	else {
		rb0 &= 0x7;
		RBS_tick_rx = rb0;
		init_RBS_tick_flag = 0;
	}

	Last_rbs_tick = rb0;

	do {
		/* no check for full ring */
		FIFO_WRITE (io_pdm_rx, dp_int_regread (reg++));
	} while (--num != 0);

	if (io_pdm_rx_wptr >= io_pdm_rx_eptr) 
		io_pdm_rx_wptr = io_pdm_rx_sptr; 

	stats.pdm_rx += num;

	return dp_int_regread (0xb0);
}

static inline unsigned char mdual_port_rx (void ) {
	extern __u16 *io_dce_rx_wptr;
	extern __u16 *io_dce_rx_sptr, *io_dce_rx_eptr;
	register __u16 *lastptr, *startptr;
	__u8 reg, rb0;
	int num;

	stats.dual_port_rx++;

	rb0 = dp_int_regread (0xb0);
	num = rb0 & 0x1f;
	if (num == 0)
		return dp_int_regread (0xb0);

	stats.rx += num;
	startptr = io_dce_rx_wptr; reg = 0x90; 
	
	while (--num != -1) {
		lastptr = io_dce_rx_wptr;

		FIFO_WRITE (io_dce_rx, dp_int_regread (reg)); reg++;
	}
	
	if (rb0 & 0x20)
		*startptr |= 0x4000;

	if (rb0 & 0x80) 
		*lastptr |= 0x8000;

	if (rb0 & 0x40) 
		*lastptr |= 0xa800;

	return dp_int_regread (0xb0);
}

#define CRL_BYTE(w)	((w) >> 8)

static inline void mdual_port_tx (void)
{
	extern __u16 *io_dce_tx_rptr, *io_dce_tx_wptr;
	extern __u16 *io_dce_tx_sptr, *io_dce_tx_eptr;
	extern __u8 dp_dsp_data_in_progress;

	unsigned char reg;
	unsigned char crl, num;

	stats.dual_port_tx++;

	dp_int_regread (0x32);

	reg = 0x80;

	crl = FIFO_CRL (io_dce_tx);
	crl &= (crl & 0x20) ? 0x20 : 0x40;

	while (reg <= 0x85 && ! FIFO_FULL (io_dce_tx)) {
		dp_int_regwrite (reg++, FIFO_DATA (io_dce_tx));
		if (FIFO_CRL (io_dce_tx) & 0x80) {
			FIFO_NEXT (io_dce_tx);
			crl |= 0x80;
			break;
		}

		FIFO_NEXT (io_dce_tx);
		if (FIFO_CRL (io_dce_tx) & 0x40) 
			break;
	}

	num = reg - 0x80; 
	if (num != 0) {
		crl |= num;
		stats.tx += num;

		dp_dsp_data_in_progress = 1;
		dp_int_regwrite (0xb0, crl);
	}
	else {
		dp_dsp_data_in_progress = 0;
		dp_int_regwrite (0xb7, 1);
	}
}

static inline void process_stream (void) {
	extern int dual_port_rx (void);
	extern void dual_port_tx (void);
	extern int io_dual_port_rx (void);
	extern unsigned short V34Mode;
	extern unsigned char S[];
	extern unsigned int DCPwIndex;
	extern unsigned char DCPBuffer[0x4b00], *DCPBufferStart; 

	if (dp_int_regread (0xb6) & 0x10) {
		if (S[0x6b] == 1 && ! V34Mode && S[0x96] == 0)
			mio_dual_port_rx();
			/* io_dual_port_rx(); */
		else
			mdual_port_rx();
	}

	if (dp_int_regread (0xb6) & 0x1) 
		mdual_port_tx();

	if (! (dp_int_regread (0xb6) & 0x20)) 
		return;

	stats.dcp++;

	unsigned char reg = 0xb3; 
	do {
		DCPBufferStart[DCPwIndex++] = dp_int_regread (reg);

		if (DCPwIndex > sizeof DCPBuffer - 1)
			DCPwIndex = 0;

	} while (--reg != 0xb0);

	dp_int_regread (0xb1);
}

static inline void process_bit3 (void) {
	extern __u8 dp_byte_c, dp_byte_d, dp_byte_e, dp_byte_f;
	dp_int_regwrite (0xd8, 0x8);

	dp_byte_c = dp_int_regread (0x32);
	dp_byte_d = dp_int_regread (0x33);
	dp_byte_e = dp_int_regread (0x34);
	dp_byte_f = dp_int_regread (0x35);
}

static inline void process_ring (void ) {
	extern __u8 dp_sleep;
	extern __u8 dp_ring_int, dp_ring_int_count, dp_polarity_int;
	extern void x_wakeup (void);
	extern void dp_init_local_phone_timer (void);
	extern unsigned char x_dsp_mars3;

	if (dp_sleep == 1) {
		dp_int_regwrite (0xcb, 0x1d);
		dp_bamil_rd7 = 0xff;
	}
	dp_ring_int_count++;

	if (x_dsp_mars3 == 4) 
		dp_int_regwrite (0xad, 0x21);

	if (! (x_dsp_mars3 == 4 && dp_ring_int_count > 1) && dp_ring_int_count <= 2) {
		dp_int_regwrite (0xd8, 0xff);
		x_wakeup ();
		if (x_dsp_mars3 == 4)
			dp_int_regwrite (0xad, 1);
	}
	else {
		dp_bamil_rd7 |= 2;
		dp_polarity_int = 0;
		dp_ring_int_count = 0;
		dp_ring_int = 1;
		dp_init_local_phone_timer();
	}
}

/* substitute for the core */
int mdp_dsp_isr (void) {
	extern void outp (__u16 addr, __u8 value);
	unsigned short BaseAddressIndex;
	unsigned char BaseValue;
	/* rd7 saved in common area and irqs from dsp disabled */

	__u8 rd7;

	/* actually it might have changed 
	 * so should it be checked or not
 	 */
	if (dp_bamil_rd7 == 0xff)
		return 0;

	rd7 = *rd7_isr;

	if (rd7 & 0x2) 
		process_ring();

	if (rd7 & 0x8) 
		process_bit3();

	if (rd7 & 0x4) {
		local_irq_disable();
		if (config.true_smp)
			mspin_lock (&Mcb->lock);

		process_stream();

		if (config.true_smp)
			mspin_unlock (&Mcb->lock);
		local_irq_enable();
	}

	atomic_dec (&Mcb->UnservedIrqs);

	/* enable interrupts */
	dp_int_regwrite (0xd7, dp_bamil_rd7);

	outp (BaseAddressIndex, BaseValue);
	return 1;
}


/* hooked */

#ifndef UART_MSR_DCD
#define UART_MSR_DCD	0x80
#endif 

#ifndef UART_MSR_DDCD
#define UART_MSR_DDCD	0x01
#endif 

void UART_cd_on() 
{
	extern void do_UART_cd_on (void ); 
	do_UART_cd_on();
}

int core_carrier_lost;
void UART_cd_off() 
{
	extern void do_UART_cd_off (void ); 
	extern __u16 io_uart_msr;
	if (io_uart_msr & UART_MSR_DCD) 
		core_carrier_lost = 1;

	do_UART_cd_off();
}

void io_init_dce_rx_fifo () {
	DECLARE_FIFO (__u16, io_dce_rx);
	extern __u16 io_dce_rx_buff[];
	extern struct martian_common *Mcb;

	io_dce_rx_sptr =
	io_dce_rx_rptr =
	io_dce_rx_wptr = io_dce_rx_buff;
	/*io_dce_rx_wptr = Mcb->io_dce_rx_buff;*/ /* in case that dce_rx should be always in common */

	/*io_dce_rx_eptr = Mcb->io_dce_rx_buff +*/
	io_dce_rx_eptr = io_dce_rx_buff +
			(sizeof Mcb->io_dce_rx_buff / sizeof Mcb->io_dce_rx_buff[0]);
}

#ifndef NULL
# define NULL 0
#endif
#define FIFO_INCORRECT(fifo)	\
({						\
	typeof (io_##fifo##_rptr) upper =	\
		 io_##fifo##_buff + (sizeof Mcb->io_##fifo##_buff / sizeof Mcb->io_##fifo##_buff[0]);	\
	io_##fifo##_rptr < io_##fifo##_buff || 	\
	io_##fifo##_rptr >= upper || 		\
	io_##fifo##_wptr < io_##fifo##_buff || 	\
	io_##fifo##_wptr >= upper;		\
}) 

void lapm_rx_hdlc_frame (void ) {
	extern void do_lapm_rx_hdlc_frame (void);

	if (Debugged (Sequence)) {
		DECLARE_FIFO (__u16, io_dce_rx);
		extern __u16 io_dce_rx_buff[];

		core.rx_hdlc++;
		if (FIFO_INCORRECT (dce_rx)) {
			core.rx_hdlc_of++;
			LOGERR ("Assertion dce_rx: buf=0x%x | rptr=0x%x, wptr=0x%x, sptr=0x%x, eptr=0x%x, lamp_rx=%d\n",
					io_dce_rx_buff, io_dce_rx_rptr, io_dce_rx_wptr, io_dce_rx_sptr, io_dce_rx_eptr, core.rx_hdlc);

			io_dce_rx_rptr = io_dce_rx_buff;
			io_dce_rx_wptr = io_dce_rx_buff;
		}

		if (io_dce_rx_sptr != io_dce_rx_buff)
			io_dce_rx_sptr = io_dce_rx_buff;
		
		if (io_dce_rx_eptr != io_dce_rx_buff +
				  (sizeof Mcb->io_dce_rx_buff / sizeof Mcb->io_dce_rx_buff[0]))
			io_dce_rx_eptr = io_dce_rx_buff +
					(sizeof Mcb->io_dce_rx_buff / sizeof Mcb->io_dce_rx_buff[0]);
	}

	do_lapm_rx_hdlc_frame();
}
