/* 	ptytosock.c: 
 *		establish connection between pty and one of martian sockets
 *
 *	usage:
 *		ptytosock [modem_symlink]
 *
 *	note:	by default /dev/modem is used for a link
 *
 *
 *	Author: A. Chentsov
 * 	Copying: LGPL
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>

#include <sys/stat.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

void msock_path_init (char *sun_path); 
int msock_path_next (char *sun_path); 

int master;

void *writer_start (void *arg) {
	int mypin = (int) arg;
	char buf[0x100];
	int bytes;

	while ( (bytes = recv (mypin, buf, 0x100, 0)) > 0) 
		write (master, buf, bytes);
	exit (1);
}

char *program;

int check_symbolic_link (char *symlink) {
	struct stat sl_stats;
	
	if (lstat (symlink, &sl_stats) == 0)
		if ( ! S_ISLNK(sl_stats.st_mode) ) {
			fprintf (stderr, "%s: %s is not a symlink\n", program, symlink);
			return 0;
		}

	return 1;
}
		
int main (int argc, char *argv[]) 
{
	char *symlink_name = "/dev/modem";

	program = argv[0];

	if (argc == 2) {
		symlink_name = argv[1];
	}
	else if (argc > 2) {
		fprintf (stderr, "Usage: %s [modemsymlink]\n", program);
		return 1;
	}

	
	master = getpt();
	if (master < 0) {
		perror ("getpt");
		return 1;
	}

	grantpt (master); unlockpt (master);

	char name[256];
	if (ptsname_r (master, name, sizeof name - 1) == 0) 
		printf ("use %s\n", name);

	int mypin = socket (AF_LOCAL, SOCK_STREAM, 0);
	if (mypin < 0) {
		fprintf (stderr, "failure to do socket()\n");
		return 1;
	}

	int res;
	struct sockaddr_un location = {
		.sun_family 	= PF_LOCAL,
	};
/*
	if (argc == 2) {
		strncpy (location.sun_path, argv[2], sizeof location.sun_path - 1);
	}
*/

	msock_path_init (location.sun_path);
	do {
		res = connect (mypin, &location, ((socklen_t)sizeof(location)));
		if (res != -1) 
			break;
	} while (msock_path_next (location.sun_path));

	if (res == -1) {
		fprintf (stderr, "failed to connect");
		return 2;
	}

	// note: symlink checking & creation is not atomic
	char *cmd;
	cmd = malloc (32 + 2 * strlen (symlink_name));
	// check symbolic link
	if (! check_symbolic_link (symlink_name))
		return 2;

	sprintf  (cmd, "rm -f %s; ln -s %s %s", symlink_name, name, symlink_name);
	system (cmd);
	printf ("%s symlink is set up,  you can start dialer\n", symlink_name);

	pthread_attr_t writer_attr;
	pthread_t writer_id;

	pthread_attr_init (&writer_attr);
	pthread_create (&writer_id, &writer_attr, writer_start, (void *)mypin); 
	// proceed reader
	char buf[0x100];
	int bytes;

	while ( (bytes = read (master, buf, 0x100)) > 0) 
		send (mypin, buf, bytes, 0);

	return 0;
}

detach () {
	/* notify waiter */
	switch (fork()) {
	case 0:
		/* offspring */
		break;
	case -1:
		/* cannot run in bg, proceed fg */
		break;

	default:
		/* starter is done */
		exit(0);
	}
}

/*** socket pool ***/
#define MARTIAN_SOCK_PATH 	"/dev/martiansock0"
#define MARTIAN_SOCK_NUM 	5

static char martian_socket_path[] = MARTIAN_SOCK_PATH;
static char *established_path = NULL;

int slot;
void msock_path_init (char *sun_path) {
	established_path = NULL;
	martian_socket_path[sizeof MARTIAN_SOCK_PATH - 2] = '0';
	strcpy (sun_path, MARTIAN_SOCK_PATH);
	slot = 0;
}

int msock_path_next (char *sun_path) {
	if (slot == MARTIAN_SOCK_NUM - 1)
		return 0;

	slot++;
	martian_socket_path[sizeof MARTIAN_SOCK_PATH - 2] = slot + '0';
	sun_path[sizeof MARTIAN_SOCK_PATH - 2] = slot + '0';
}

void msock_path_establish () {
	established_path = martian_socket_path;
}

